import whatap.agent.conf.configure as conf
import whatap.util.logging_util as logging_util
import sys

class TaskConfig:
    def __init__(self):
	    self.process(None, None, None, None)

    def interval(self):
        return 15

    def process(self, os_ , pcode , oid , now ) :
        conf.reload()
        logging_util.isDebug = conf.GetConfig().Debug

        if not conf.GetConfig().Enabled:
            logging_util.info("Received Agent Status Closed received. Shutting down...")
            sys.exit(0)

        return None
