Configuration = {
    "enabled": ('Enabled',True),
    "license": ('License',None),
    "whatap.server.host": ('WhatapHost', ''),
    "whatap.server.port":  ('WhatapPort', 6600),
    "cypher_level":  ('CypherLevel',128),
    "EncryptLevel": 0,

    "time_sync_interval": ('TimeSyncInterval',30),

    "downcheck.listen.enabled":    ('DownCheckListenEnabled', False),
    "downcheck.enabled":    ('DownCheckEnabled', False),
    "downcheck.interval":   ('DownCheckInterval',0),
    "downcheck.timeout":    ('DownCheckTimeout',0),
    "downcheck.concurrent": ('DownCheckConcurrent',0),
    "downcheck.target":  ('DownCheckTargetStr',""),
    "downcheck.concurrent": ('DownCheckConcurrent', 0),
    "peer.port": ('PeerPort', 10120),

    "createdtime":         ('CreatedTime',0),

    "tag.agent": ('Tags',''),
    "sample_type": ('SampleType',None),

    "execution.cache.timeout": ('ExecutionScriptCacheTimeout', 60000),

    "debug": ('Debug',False),

    "log.line.limit": ('LogLineLimit', 60),
    "file.count.interval": ('FileCountInterval', 60),

    "housekeeping.enabled":('HouseKeepEnabled', True),
    "housekeeping.hour": ('HouseKeepHour', 2),
	"housekeeping.cooltime": ('HouseKeepCooltime', 12),
    "healthcheck.cooltime": ('HealthCheckCoolTime', 3),
    "agentless.interval": ("AgentlessInterval", 60000),
    "agentless.timeout":("AgentlessTimeout", 10000),
    "agentless.concurrent":('AgentlessConcurrent', 5),
    "agentless.items":('AgentlessItems', []),

    "sunos.memstat.enabled":("SunosMdbEnabled", False),
    "sunos.memstat.interval": ("MemstatInterval", 30),
    "sunos.disk.quota.enabled": ("SunosDiskQuotaEnabled", False),
    "aix.diskstat.interval": ("DiskstatInterval", 5),
    "process.fdcheck":('serverProcessFDCheck', False),
    "process.connectcheck":('serverProcessConnectCheck', False),
    "process.moncheck": ("serverProcessMonCheck", True),
    "tcpcheck_timeout": ("tcpcheck_timeout", 4),
    "process_update_interval": ("process_update_interval", 20),
    "oid": ("oid",0),
    "oname": ("oname", ''),

    "task_timeout_except": ("task_timeout_except", True),
    "task_timeout": ("task_timeout", 2),
    "task_health_check_count": ("task_health_check_count", 5),
    "task_disk": ("task_disk", True),
    "disk.io.enabled": ("DiskIOEnabled", True),
    "task_osinfo": ("task_osinfo", True),
    "task_base": ("task_base", True),
    "task_net": ("task_net", True),
    "task_proc": ("task_proc", True),
    "task_config": ("task_config", True),
    "task_filesystem": ("task_filesystem", True),
    "task_tcpcheck": ("task_tcpcheck", True),
    "task_logevent": ("task_logevent", True),

    "hostinfov2.enabled":("HostInfoV2Enabled", False),
    "categoryv1.enabled":("CategoryV1Enabled", True),
    "categoryv2.enabled":("CategoryV2Enabled", False),

    "categoryv2.interval":("CategoryV2Interval", 2),
    "categoryv2.folding.interval":("CategoryV2FoldingInterval", 5),

    "ipreg":("ipreg", False),
    "log.size":("logSize", 4), 
    "log.backup":("logBackup", 3),
    "log.path":("logPath", "/var/log"),

    "aix.errpt.enabled":("ErrptReditectEnabled", True),
    "aix.errpt.Interval":("ErrptReditectInterval", 60),
    "aix.errpt.log.path":("ErrptLogPath", "/var/log/whatap_errpt_message.log"),
    "aix.errpt.log.size":("ErrptLogSize", 4),
    "aix.errpt.log.backup":("ErrptLogFileCount", 3),

    "package.dir":("PackageDir", "/tmp/whatap-package"),

    "cronjob.enabled":("CronjobEnabled", False),
    "cronjob.path":("CronjobPath", "/usr/whatap/infra/cronjob"),
    "cronjob.conf":("CronjobConf", "cronjob.json"),
    "cronjob.timeout":("CronjobTimeout", 5),

}
