
import platform

_getCommandList = None
_execute = None

system = platform.system().lower()
if 'sunos' == system:
    import remotecommand_sunos as remotecmd
    _getCommandList = remotecmd.getCommandList
    _execute = remotecmd.execute

def getCommandList():
    return _getCommandList()


def execute(scriptSlot , args ):
    return _execute(scriptSlot , args)

