from whatap.value.value import Value


class SummaryValue(Value):
    def __init__(self):
        super(SummaryValue, self).__init__()

    def add(self, value):
        return self

    def addCount(self):
        return

    def longSum(self):
        return

    def longMin(self):
        return

    def longMax(self):
        return

    def longAvg(self):
        return

    def doubleSum(self):
        return

    def doubleMin(self):
        return

    def doubleMax(self):
        return

    def doubleAvg(self):
        return

    def getCount(self):
        return
